#include <avr/io.h>
#include <avr/pgmspace.h>
#include <util/delay.h>

#define BITLEN 32	// ms
#define PREAMBLE 200	// bits
#define TAIL 200	// bits

#include "sinetable.h"
#include "message.h"

// Note: some of the code here is deliberately inefficient to minimise timing jitter.

unsigned char phase = 0;	// current phase in LSB

void init() {
	// Setup OC0A and OC0B as outputs in fast PWM mode with no IO clock prescaler
	TCCR0A = _BV(COM0A1) | _BV(COM0B1) | _BV(WGM01) | _BV(WGM00);
	TCCR0B = _BV(CS00);
	
	// setup OC0A and OC0B and PB2 for output:
	//
	//	PB0 is baseband PSK31 signal	(analogue)
	//	PB1 is the amplitude envelope	(analogue)
	//	PB2 is the carrier phase	(digital)
	//
	DDRB = _BV(PB0) | _BV(PB1) | _BV(PB2);
}

void sendBit(int bit) {
	int i, idx;
	for(i = 0; i < SAMPLES; i++) {
		idx = (phase)?(i):(SAMPLES - i - 1);
		if(!bit) {
			OCR0A = pgm_read_byte(&basebandData[idx]);
			OCR0B = pgm_read_byte(&envelopeData[idx]);
			if(OCR0B == 0) PORTB ^= _BV(PB2);
		}
		_delay_ms(BITLEN/SAMPLES);
	}

	if(!bit) phase ^= 1;
}

void sendMessage() {
	int i;
	unsigned char byte;
	
	for(i = 0; i < LENGTH*8; i++) {
		byte = pgm_read_byte(&varicodeData[i/8]);
		sendBit(byte & (1 << (7 - (i%8))));
	}
}

int main() {
	unsigned char i = 0;

	init();
	while(1) {
		for(i = 0; i < PREAMBLE; i++) sendBit(0);
		sendMessage();
		for(i = 0; i < TAIL; i++) sendBit(1);
	}
	
	return 0;  // unreached
}
